<?php

namespace Drupal\micro_simple_sitemap;

use Drupal\Core\Cache\Cache;
use Drupal\Core\State\StateInterface;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\micro_site\SiteNegotiatorInterface;
use Drupal\simple_sitemap\Batch ;

/**
 * Batch class.
 */
class MicroBatch extends Batch {

  /**
   * The micro site negotiator.
   *
   * @var \Drupal\micro_site\SiteNegotiatorInterface
   */
  protected $negotiator;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Batch constructor.
   *
   * @param \Drupal\micro_site\SiteNegotiatorInterface $micro_site_negotiator
   * @param \Drupal\Core\State\StateInterface $state
   */
  public function __construct(SiteNegotiatorInterface $micro_site_negotiator, StateInterface $state) {
    parent::__construct();
    $this->negotiator = $micro_site_negotiator;
    $this->state = $state;
  }

  /**
   * Starts the batch process depending on where it was requested from.
   */
  public function start() {
    switch ($this->batchSettings['from']) {

      case 'form':
        // Start batch process.
        batch_set($this->batch);
        return TRUE;

      case 'drush':
        // Start drush batch process.
        batch_set($this->batch);

        // See https://www.drupal.org/node/638712
        $this->batch =& batch_get();
        $this->batch['progressive'] = FALSE;

        drush_log($this->t(self::BATCH_INIT_MESSAGE), 'status');
        drush_backend_batch_process();
        return TRUE;

      case 'backend':
        // Start backend batch process.
        batch_set($this->batch);

        // See https://www.drupal.org/node/638712
        $this->batch =& batch_get();
        $this->batch['progressive'] = FALSE;

        // todo: Does not take advantage of batch API and eventually runs out of memory on very large sites. Use queue API instead?
        batch_process();
        return TRUE;

      case 'nobatch':
        // Call each batch operation the way the Drupal batch API would do, but
        // within one process (so in fact not using batch API here, just
        // mimicking it to avoid code duplication).
        $context = [];
        foreach ($this->batch['operations'] as $i => $operation) {
          $operation[1][] = &$context;
          call_user_func_array($operation[0], $operation[1]);
        }
        $this->finishGeneration(TRUE, !empty($context['results']) ? $context['results'] : [], [], isset($this->batchSettings['site']) ? $this->batchSettings['site'] : NULL);
        return TRUE;

      case 'micro_site':
        // Call each batch operation the way the Drupal batch API would do, but
        // within one process (so in fact not using batch API here, just
        // mimicking it to avoid code duplication).
        $context = [];
        foreach ($this->batch['operations'] as $i => $operation) {
          $operation[1][] = &$context;
          call_user_func_array($operation[0], $operation[1]);
        }
        $this->finishGeneration(TRUE, !empty($context['results']) ? $context['results'] : [], [], isset($this->batchSettings['site']) ? $this->batchSettings['site'] : NULL);
        return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function finishGeneration($success, $results, $operations, SiteInterface $site = NULL) {
    if ($success) {
      $remove_sitemap = empty($results['chunk_count']);
      if (!empty($results['generate']) || $remove_sitemap) {
        \Drupal::service('micro_simple_sitemap.sitemap_generator')
          ->generateSitemap($results['generate'], $remove_sitemap, $site);
      }
      if ($site instanceof SiteInterface) {
        Cache::invalidateTags(['micro_simple_sitemap:' . $site->id()]);
      }
      else {
        Cache::invalidateTags(['simple_sitemap']);
      }

      \Drupal::service('simple_sitemap.logger')->m(self::REGENERATION_FINISHED_MESSAGE,
        ['@url' => $GLOBALS['base_url'] . '/sitemap.xml'])
        ->display('status')
        ->log('info');
    }
    else {
      \Drupal::service('simple_sitemap.logger')->m(self::REGENERATION_FINISHED_ERROR_MESSAGE)
        ->display('error', 'administer sitemap settings')
        ->log('error');
    }
  }

}
