<?php

namespace Drupal\micro_simple_sitemap;

use Drupal\Component\Datetime\Time;
use Drupal\Core\State\StateInterface;
use Drupal\micro_site\Entity\Site;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\micro_site\SiteNegotiatorInterface;
use Drupal\simple_sitemap\EntityHelper;
use Drupal\simple_sitemap\Simplesitemap;
use Drupal\simple_sitemap\SitemapGenerator;
use Drupal\simple_sitemap\SitemapWriter;
use XMLWriter;
use Drupal\simple_sitemap\Batch;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Class MicroSimpleSitemapGenerator.
 *
 * @package Drupal\micro_simple_sitemap
 */
class MicroSimpleSitemapGenerator extends SitemapGenerator {

  /**
   * The micro site negotiator.
   *
   * @var \Drupal\micro_site\SiteNegotiatorInterface
   */
  protected $negotiator;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * SitemapGenerator constructor.
   *
   * @param \Drupal\simple_sitemap\EntityHelper $entityHelper
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   * @param \Drupal\Component\Datetime\Time $time
   * @param \Drupal\simple_sitemap\SitemapWriter $sitemapWriter
   * @param \Drupal\micro_site\SiteNegotiatorInterface $micro_site_negotiator
   * @param \Drupal\Core\State\StateInterface $state
   */
  public function __construct(EntityHelper $entityHelper, Connection $database, ModuleHandler $module_handler, LanguageManagerInterface $language_manager, Time $time, SitemapWriter $sitemapWriter, SiteNegotiatorInterface $micro_site_negotiator, StateInterface $state) {
    parent::__construct($entityHelper, $database, $module_handler, $language_manager, $time, $sitemapWriter);
    $this->negotiator = $micro_site_negotiator;
    $this->state = $state;
  }

  public function generateSitemap(array $links, $remove_sitemap = FALSE, SiteInterface $site = NULL) {
    if (!$site instanceof SiteInterface) {
      parent::generateSitemap($links, $remove_sitemap);
      return;
    }

    $sitemap_string = $this->generateSitemapChunk($links);
    $this->state->set('micro_simple_sitemap:' . $site->id(), $sitemap_string);
  }


}
